#pragma once

#include <stdint.h>
#include <sstream>
#include <boost/date_time.hpp>

#pragma pack(1)

namespace knx {

class dpt
{
public:
    template<typename data_type>
    static std::string describe(const typename data_type::major_type::set_type & data);
};

/**
 * @brief major class for all 1.XXX data point types
 */
class dpt_1 {
public:
    typedef uint8_t data_type;
    typedef bool set_type;
    static void set(set_type value, data_type & data);
    static set_type get(const data_type & data);
};

/**
 * \brief data point type 1.001 simple boolean value
 */
class dpt_1_001
{
public:
    typedef dpt_1 major_type;
    static const major_type::set_type ON = true;
    static const major_type::set_type OFF = false;
    static std::string describe(const major_type::set_type & data)
    {
        if (data)
            return "on";
        return "off";
    }
};

//! data point type 1.001 is better known as switch
typedef dpt_1_001 dpt_switch;

/**
 * @brief major class for all 5.XXX data point types
 */
class dpt_5 {
public:
    typedef uint16_t data_type;
    typedef uint8_t set_type;
    static void set(set_type value, data_type & data);
    static set_type get(const data_type & data);
};

/**
 * \brief data point type 5.001 scaling value
 */
class dpt_5_001
{
public:
    typedef dpt_5 major_type;
    static std::string describe(const major_type::set_type & data)
    {
        std::stringstream ss;
        ss << data << "%";
        return ss.str();
    }
};

//! 5.001 is better known as scaling value
typedef dpt_5_001 dpt_scaling;

/**
 * @brief major class for all 10.XXX data point types
 */
class dpt_10 {
public:
    //! @brief internal class storing a time value
    class time {
    public:
        //! @brief creates a current time value
        time();
        uint8_t day;
        uint8_t hour;
        uint8_t minutes;
        uint8_t seconds;
    };
//     static time now(); TODO
    typedef uint32_t data_type;
    typedef time set_type;
    static void set(set_type value, data_type & data);
    static set_type get(const data_type & data);
};

/**
 * @brief class holding a time value
 */
class dpt_10_001
{
public:
    typedef dpt_10 major_type;
    static std::string describe(const major_type::set_type & data)
    {
        std::stringstream ss;
        ss << data.hour << ":" << data.minutes << ":" << data.seconds;
        return ss.str();
    }
    static const uint8_t NODAY = 0;
    static const uint8_t MONDAY = 1;
    static const uint8_t TUESDAY = 2;
    static const uint8_t WEDNESDAY = 3;
    static const uint8_t THURSDAY = 4;
    static const uint8_t FRIDAY = 5;
    static const uint8_t SATURDAY = 6;
    static const uint8_t SUNDAY = 7;
};

//! 10.001 is known as time of day value including hours. minutes, seconds and day of the week
typedef dpt_10_001 dpt_timeofday;







}

#pragma pack()


